import uuid

from azure.common.credentials import ServicePrincipalCredentials
from . import exceptions
from .records import AzureClient

from typing import List, Any, Optional, Callable

from ppa import ui


def service_principal_credentials(
    client: AzureClient, resource="https://management.core.windows.net/"
) -> ServicePrincipalCredentials:
    if missing_keys := [key for key in ["client_id", "client_secret", "tenant_id"] if key not in client]:
        raise exceptions.MissingApiSecret("The following keys are missing from azure_client: {}".format(" ,".join(missing_keys)))

    return ServicePrincipalCredentials(
        client_id=client["client_id"], secret=client["client_secret"], tenant=client["tenant_id"], resource=resource
    )


def get_table_field(dictionary, field):
    return str(dictionary.get(field)) if dictionary.get(field) is not None else ""


def input_table(
    text: str,
    data: List[Any],
    header: List[str],
    fields: List[Callable],
    minimum: Optional[int] = None,
    maximum: Optional[int] = None,
) -> List[Any]:

    table_element_id = str(uuid.uuid4())
    error_element_id = str(uuid.uuid4())
    while True:
        indices = ui.input_table(
            text, header=header, data=[[get(item) for get in fields] for item in data], element_id=table_element_id,
        )
        count = len(indices)
        if minimum and count < minimum:
            ui.output_error(f"Please choose at least {minimum} record(s)", element_id=error_element_id)
            continue
        if maximum and count > maximum:
            ui.output_error(f"Please choose no more than {maximum} record(s)", element_id=error_element_id)
            continue
        break

    return [data[i] for i in indices]
